//////////////////////////////////////////////////////////////////////////////
//
//  NI FlexRIO Interface C API
//
//  Copyright : National Instruments 2012.  All Rights Reserved.
//  Purpose   : Public header file for the niflexrio library.
//
//////////////////////////////////////////////////////////////////////////////
#ifndef niflexrio_niflexrio_h
#define niflexrio_niflexrio_h


// Windows and Linux are currently the only officially supported
// operating systems.  Other OSs may or may not work, but have not been tested.
// These #defines may need to be modified for other OSs.
//
#if defined(linux) || defined(__linux)
#   define NIFLEXRIO_EXPORT __attribute__ ((section (".export")))
#   define NIFLEXRIO_CDECL
#elif defined(__APPLE__)
#   define NIFLEXRIO_EXPORT __attribute__((visibility("default")))
#   define NIFLEXRIO_CDECL
#else
#   define NIFLEXRIO_EXPORT __declspec(dllexport)
#   define NIFLEXRIO_CDECL  __cdecl
#endif


// Define exact-width integer types, if they have not already been defined.
//
#if NiFlexRio_ExactWidthIntegerTypesDefined \
    || defined(_STDINT) \
    || defined(_STDINT_H) \
    || defined(_STDINT_H_) \
    || defined(_INTTYPES_H) \
    || defined(_INTTYPES_H_) \
    || defined(_SYS_STDINT_H) \
    || defined(_SYS_STDINT_H_) \
    || defined(_SYS_INTTYPES_H) \
    || defined(_SYS_INTTYPES_H_) \
    || defined(_STDINT_H_INCLUDED) \
    || defined(BOOST_CSTDINT_HPP) \
    || defined(_MSC_STDINT_H_) \
    || defined(_PSTDINT_H_INCLUDED)
    // Assume that exact-width integer types have already been defined.
#elif defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L \
    || defined(__GNUC__) /* GCC (at least since 3.0) has a stdint.h. */ \
    || defined(HAVE_STDINT_H)
    // Assume that stdint.h can be included.
    #include <stdint.h>
#elif defined(_CVI_) \
    || defined(_MSC_VER)
    // Manually define exact-width integer types.
    typedef   signed    char  int8_t;
    typedef unsigned    char uint8_t;
    typedef            short  int16_t;
    typedef unsigned   short uint16_t;
    typedef              int  int32_t;
    typedef unsigned     int uint32_t;
    typedef          __int64  int64_t;
    typedef unsigned __int64 uint64_t;
#else
    // Exact-width integer types must be defined by the user, and the following
    // macro must be #defined, before #including niflexrio.h:
    //
    //    #define NiFlexRio_ExactWidthIntegerTypesDefined 1
    //
    #error Exact-width integer types must be defined by the user. See comment.
#endif


//============================================================================
//  Attributes - IMPORTANT:  The numeric codes of these attributes need to
//      match the numeric codes of the attributes in the FlexRIO_Host_Attribute.ctl
//      typdef in our LabVIEW host interface.
//============================================================================
typedef enum {
    NIFLEXRIO_Attr_MemGuard                   = 0xFFFFFFFF,
    NIFLEXRIO_Attr_FamPresent                 = 0,
    NIFLEXRIO_Attr_FamPowerGood               = 1,
    NIFLEXRIO_Attr_VccoProgrammedSuccessfully = 2,
    NIFLEXRIO_Attr_PXIClk10Present            = 3,
    NIFLEXRIO_Attr_FamIOEnabled               = 4,
    NIFLEXRIO_Attr_FamPowerEnabled            = 5,
    NIFLEXRIO_Attr_EEPROMPowerEnabled         = 6,
    NIFLEXRIO_Attr_FamPowerGoodTimeout        = 7,
    NIFLEXRIO_Attr_FamIDMismatch              = 8,
    NIFLEXRIO_Attr_InsertedFamID              = 9,
    NIFLEXRIO_Attr_ExpectedFamID              = 10,
    NIFLEXRIO_Attr_CurrentTemperature         = 11,
    NIFLEXRIO_Attr_VccoARaw                   = 12,
    NIFLEXRIO_Attr_VccoBRaw                   = 13,
    NIFLEXRIO_Attr_Signature                  = 14,
    NIFLEXRIO_Attr_Revision                   = 15,
    NIFLEXRIO_Attr_OldestCompatibleRevision   = 16,
    NIFLEXRIO_Attr_FamState                   = 17,
    NIFLEXRIO_Attr_I2CMux                     = 18,
    NIFLEXRIO_Attr_InsertedFamHasEEPROM       = 19,
    NIFLEXRIO_Attr_ExpectedFamHasEEPROM       = 20,
    NIFLEXRIO_Attr_IsFamSupported             = 21,
    NIFLEXRIO_Attr_FamIDReadTimeout           = 22,
    NIFLEXRIO_Attr_PXIeClk100Locked           = 23,
    NIFLEXRIO_Attr_SerialNum                  = 24,
    NIFLEXRIO_Attr_FamSerialNum               = 25,
    NIFLEXRIO_Attr_FanSpeed                   = 26,
    NIFLEXRIO_Attr_FanPWM                     = 27,
    NIFLEXRIO_Attr_END
} NIFLEXRIO_Attr;

//============================================================================
//  Value Types
//============================================================================
typedef enum {
    NIFLEXRIO_ValueType_MemGuard   = 0xFFFFFFFF,
    NIFLEXRIO_ValueType_U32        = 0,
    NIFLEXRIO_ValueType_END
} NIFLEXRIO_ValueType;

//============================================================================
//  Terminal Block Manager States - IMPORTANT:  Do NOT change the enumerated
//      values of these states.  They must match the values in hardware.
//============================================================================
typedef enum {
    NIFLEXRIO_FamState_MemGuard       = 0xFFFFFFFF,
    NIFLEXRIO_FamState_Idle           = 0,
    NIFLEXRIO_FamState_NoFam          = 1,
    NIFLEXRIO_FamState_EnableEEPROM   = 2,
    NIFLEXRIO_FamState_CheckFamID     = 3,
    NIFLEXRIO_FamState_EnableFamPower = 4,
    NIFLEXRIO_FamState_EnableFamIO    = 5,
    NIFLEXRIO_FamState_FamAlive       = 6,
    NIFLEXRIO_FamState_FamDead        = 7,
    NIFLEXRIO_FamState_SWAccessEEPROM = 8
} NIFLEXRIO_FamState;

//============================================================================
//  I2C Mux Select - IMPORTANT:  Do NOT change the enumerated values.  They
//      must match the values in hardware.
//============================================================================
typedef enum {
    NIFLEXRIO_I2CMux_MemGuard = 0xFFFFFFFF,
    NIFLEXRIO_I2CMux_None     = 0,
    NIFLEXRIO_I2CMux_Hardware = 1,
    NIFLEXRIO_I2CMux_Software = 2,
    NIFLEXRIO_I2CMux_LVFPGA   = 3
} NIFLEXRIO_I2CMux;

//============================================================================
//  I2C Command
//============================================================================
typedef enum {
    NIFLEXRIO_I2CCmd_MemGuard = 0xFFFFFFFF,
    NIFLEXRIO_I2CCmd_Write    = 0,
    NIFLEXRIO_I2CCmd_Read     = 1
} NIFLEXRIO_I2CCmd;

//============================================================================
//  I2C Access Control
//============================================================================
typedef enum {
    NIFLEXRIO_I2CAccess_MemGuard = 0xFFFFFFFF,
    NIFLEXRIO_I2CAccess_Acquire  = 0,
    NIFLEXRIO_I2CAccess_Release  = 1
} NIFLEXRIO_I2CAccess;

//============================================================================
//  Shutdown Limit Types
//============================================================================
typedef enum {
    NIFLEXRIO_ShutdownLimit_MemGuard   = 0xFFFFFFFF,
    NIFLEXRIO_ShutdownLimit_Thermal    = 0,
    NIFLEXRIO_ShutdownLimit_TotalPower = 1,
    NIFLEXRIO_ShutdownLimit_12vPower   = 2,
    NIFLEXRIO_ShutdownLimit_3v3Power   = 3
} NIFLEXRIO_ShutdownLimit;

//============================================================================
//  Error/Warning Types
//============================================================================
typedef enum {
    NIFLEXRIO_DeviceStatus_MemGuard                     = 0xFFFFFFFF,
    NIFLEXRIO_DeviceStatus_Normal                       = 0,
    NIFLEXRIO_DeviceStatus_ConfigVoltageMonitorsFailed  = 1,
    NIFLEXRIO_DeviceStatus_ReadVoltageMonitorsFailed    = 2,
    NIFLEXRIO_DeviceStatus_ConfigPowerMonitorsFailed    = 3,
    NIFLEXRIO_DeviceStatus_ReadPowerMonitorsFailed      = 4,
    NIFLEXRIO_DeviceStatus_ReadConfigEEPROMFailed       = 5,
    NIFLEXRIO_DeviceStatus_FanSlowed                    = 6,
    NIFLEXRIO_DeviceStatus_FanStopped                   = 7,
    NIFLEXRIO_DeviceStatus_PLLLossofLock                = 8,
    NIFLEXRIO_DeviceStatus_Rail2v5AlarmTripped          = 9,
    NIFLEXRIO_DeviceStatus_ConfigAD9511Failed           = 10,
    NIFLEXRIO_DeviceStatus_ConfigFpgaTempMonitorFailed  = 11,
    NIFLEXRIO_DeviceStatus_ReadFpgaTempMonitorFailed    = 12,
    NIFLEXRIO_DeviceStatus_ThermalShutdown              = 13,
    NIFLEXRIO_DeviceStatus_PowerShutdown                = 14,
    NIFLEXRIO_DeviceStatus_FixedLogicVersionMismatch    = 15,
    NIFLEXRIO_DeviceStatus_USBOverCurrent               = 16,
    NIFLEXRIO_DeviceStatus_SDOverCurrent                = 17
} NIFLEXRIO_DeviceStatus;



//============================================================================
//  Begin API Functions
//============================================================================
#if ( defined(__cplusplus) )
extern "C" {
#endif


//! Query an attribute for a NI FlexRIO device.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] attribute The attribute to query.  Please refer to, and use, the enum NIFLEXRIO_Attr.
//! \param [in] valueType The type of attribute to query.  Please refer to, and use, the enum NIFLEXRIO_ValueType.
//! \param [out] value The result will be placed in this variable.  The client is responsible for proper memory allocation/deallocation.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_GetAttribute(uint32_t const deviceHandle, int32_t const attribute, int32_t const valueType, void * const value);


//! Query an array of attributes for a NI FlexRIO device.  For multiple attributes, querying as an array will be significantly faster than individually.  The attributes will be queried in the order they appear in the array.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] attributeArray The attributes to query.  Please refer to, and use, the enum NIFLEXRIO_Attr.
//! \param [in] valueTypeArray The types of attributes to query.  Please refer to, and use, the enum NIFLEXRIO_ValueType.
//! \param [in] arraySize The number of attributes to be queried.
//! \param [out] valueArray The results will be placed in this variable.  The client is responsible for proper memory allocation/deallocation.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_GetAttributesArray(uint32_t const deviceHandle, int32_t const * const attributeArray, int32_t const * const valueTypeArray, uint32_t const arraySize, void * const valueArray);


//! Control and NI FlexRIO device's adapter module's power.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] enable Control the FAM power by enabling (1) or disabling (0).
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamControlPower(uint32_t const deviceHandle, uint32_t const enable);


//! Read from a NI FlexRIO's adapter module's EEPROM.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] address The address in the EEPROM to begin reading from.
//! \param [in] numBytes The number of bytes to read.
//! \param [out] data The results will be placed in this variable.  The client is responsible for proper memory allocation/deallocation.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamReadEeprom(uint32_t const deviceHandle, uint8_t const address, uint16_t const numBytes, uint8_t * const data);


//! Write to a NI FlexRIO's adapter module's EEPROM.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] address The address in the EEPROM to begin writing to.
//! \param [in] numBytes The number of bytes to write.
//! \param [in] data The data to write into the EEPROM will be taken from this variable.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamWriteEeprom(uint32_t const deviceHandle, uint8_t const address, uint16_t const numBytes, uint8_t const * const data);


//! Control software's access to the I2C bus for the adapter module.  Access must be acquired prior to calling NiFlexRio_FamIssueI2CBusCycle(), and must be released afterwards.
//! \param [in] deviceHandle The device handle for a FlexRIO device returned from NiFpga_Open().
//! \param [in] i2cAccessMethod The type of access to request.  Please refer to, and use, the enum NIFLEXRIO_I2CAccess.
//! \param [in] timeout The amount of time in milliseconds to wait before the command times out.  Use -1 to wait indefinitely.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamI2CAccessControl(uint32_t const deviceHandle, int32_t const i2cAccessMethod, int32_t const timeout);


//! Issue an I2C command to the NI FlexRIO adapter module.  IMPORTANT: Software access to the I2C bus *must* be acquired *prior* to issuing this command.  Use NiFlexRioLV_FamI2CAccessControl() above.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] i2cCommand The I2C command for the device.  Please refer to, and use, the enum NIFLEXRIO_I2CCmd.
//! \param [in] startBit Issue a start bit?  1 for yes, 0 for no.
//! \param [in] stopBit Issue a stop bit?  1 for yes, 0 for no.
//! \param [in] expectAck Will the other device acknowledge us and ACK back?  1 for yes, 0 for no.
//! \param [in] writeData The data to write to the bus (not used for reads).
//! \param [out] readData The data read from the bus (not used for writes).
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamIssueI2CBusCycle(uint32_t const deviceHandle, int32_t const i2cCommand, uint32_t const startBit, uint32_t const stopBit, uint32_t const expectAck, uint8_t const writeData, uint8_t * const readData);


//! Set the product ID for the device's adapter module.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] productId Set the product ID for this adapter module.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamSetProductId(uint32_t const deviceHandle, uint32_t const productId);


//! Get the product ID for the device's adapter module.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [out] productId The product ID read from the device will be placed in this variable.  The client is responsible for proper memory allocation/deallocation.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamGetProductId(uint32_t const deviceHandle, uint32_t * const productId);


//! Redetect the device's adapter module.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FamRedetect(uint32_t const deviceHandle);


//! Read a user attribute from the device.  The attributes are non-volatile, for client use, and may be used however a client pleases.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] attribute The attribute to read.
//! \param [out] value The attribute will be placed into this variable after it is read.  The client is responsible for proper memory allocation/deallocation.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_ReadUserAttribute(uint32_t const deviceHandle, uint32_t const attribute, uint32_t * const value);


//! Write a user attribute to the device.  The attributes are non-volatile, for client use, and may be used however a client pleases.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] attribute The attribute to write.
//! \param [in] value The attribute will be set to the value of this variable.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_WriteUserAttribute(uint32_t const deviceHandle, uint32_t const attribute, uint32_t const value);

//! Set the temperature or power usage at which the device will shut down to prevent damage.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] type The type of shutdown limit to set.  Please refer to, and use, the enum NIFLEXRIO_ShutdownLimit.
//! \param [in] value The limit will be set to the value of this variable.  Temperature is in degrees C, power is in milliwatts.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_SetDeviceShutdownLimit(uint32_t const deviceHandle, int32_t const type, uint32_t const value);

//! Get the temperature or power usage at which the device will shut down to prevent damage.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [in] type The type of shutdown limit to get.  Please refer to, and use, the enum NIFLEXRIO_ShutdownLimit.
//! \param [out] value The attribute will be placed into this variable after it is read.  Temperature is in degrees C, power is in milliwatts.  The client is responsible for proper memory allocation/deallocation.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_GetDeviceShutdownLimit(uint32_t const deviceHandle, int32_t const type, uint32_t * const value);

//! Get the current state of the device.
//! \param [in] deviceHandle The device handle for a NI FlexRIO device returned from NiFpga_Open().
//! \param [out] value The value of the most severe status of the device will be placed in this variable.
//! \return status code
int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_GetDeviceStatus(uint32_t const deviceHandle, int32_t * const value);


//
// Deprecated functions
//

// This function has been deprecated.  Please use NiFlexRio_ReadAttribute() instead.
//int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FlashGetAttribute(uint32_t deviceHandle, uint32_t attributeOffset, uint32_t * value);

// This function has been deprecated.  Please use NiFlexRio_WriteAttribute() instead.
//int32_t NIFLEXRIO_EXPORT NIFLEXRIO_CDECL NiFlexRio_FlashSetAttribute(uint32_t deviceHandle, uint32_t attributeOffset, const uint32_t * value);


#if ( defined(__cplusplus) )
}
#endif
//============================================================================
//  End API Functions
//============================================================================

#endif
